"use strict";

var fs = require("fs");

var _require = require("crypto"),
    createHash = _require.createHash;

var path = require("path");
var _ = require("lodash");
var mkdirp = require("mkdirp");
var chalk = require("chalk");
var filesize = require("filesize");
var moment = require("moment");
var debug = require("simple-node-logger").createSimpleLogger();

function removeFile(filePath) {
  setTimeout(function () {
    if (filePath.indexOf("hot-update") !== -1) {
      fs.unlink(filePath, function (err) {
        if (err) throw err;
      });
    }
  }, 200);
}

/**
 * When 'webpack' program is used, constructor name is equal to 'NodeOutputFileSystem'.
 * When 'webpack-dev-server' program is used, constructor name is equal to 'MemoryFileSystem'.
 */
var isMemoryFileSystem = function isMemoryFileSystem(outputFileSystem) {
  return outputFileSystem.constructor.name === "MemoryFileSystem";
};

module.exports = function WriteFileWebpackPlugin() {
  var userOptions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var options = _.assign({}, {
    exitOnErrors: true,
    force: false,
    log: false,
    test: null,
    useHashIndex: true,
    outputPath: "/"
  }, userOptions);

  if (!_.isBoolean(options.exitOnErrors)) {
    throw new TypeError("options.exitOnErrors value must be of boolean type.");
  }

  if (!_.isBoolean(options.force)) {
    throw new TypeError("options.force value must be of boolean type.");
  }

  if (!_.isBoolean(options.log)) {
    throw new TypeError("options.log value must be of boolean type.");
  }

  if (!_.isNull(options.test) && !(_.isRegExp(options.test) || _.isFunction(options.test))) {
    throw new TypeError("options.test value must be an instance of RegExp or Function.");
  }

  if (!_.isBoolean(options.useHashIndex)) {
    throw new TypeError("options.useHashIndex value must be of boolean type.");
  }

  var log = function log() {
    for (var _len = arguments.length, append = Array(_len), _key = 0; _key < _len; _key++) {
      append[_key] = arguments[_key];
    }

    if (!options.log) {
      return;
    }

    debug.info.apply(debug, [chalk.dim("[" + moment().format("HH:mm:ss") + "] [write-file-webpack-plugin]")].concat(append));
  };

  var assetSourceHashIndex = {};

  log("options", options);

  var apply = function apply(compiler) {
    var outputPath = void 0;
    var setupDone = void 0;
    var setupStatus = void 0;

    var setup = function setup() {
      if (setupDone) {
        return setupStatus;
      }

      setupDone = true;

      log('compiler.outputFileSystem is "' + chalk.cyan(compiler.outputFileSystem.constructor.name) + '".');

      if (!isMemoryFileSystem(compiler.outputFileSystem) && !options.force) {
        return false;
      }

      outputPath = options.outputPath;
      if (!outputPath) {
        throw new Error("outputPath is not defined. Define outputPath.");
      }

      log('outputPath is "' + chalk.cyan(outputPath) + '".');

      setupStatus = true;

      return setupStatus;
    };

    // eslint-disable-next-line promise/prefer-await-to-callbacks
    var handleAfterEmit = function handleAfterEmit(compilation, callback) {
      if (!setup()) {
        // eslint-disable-next-line promise/prefer-await-to-callbacks
        callback();

        return;
      }

      if (options.exitOnErrors && compilation.errors.length) {
        // eslint-disable-next-line promise/prefer-await-to-callbacks
        callback();

        return;
      }

      log('compilation.errors.length is "' + chalk.cyan(compilation.errors.length) + '".');

      _.forEach(compilation.assets, function (asset, assetPath) {
        var outputFilePath = path.isAbsolute(assetPath) ? assetPath : path.join(outputPath, assetPath);
        var relativeOutputPath = path.relative(process.cwd(), outputFilePath);
        var targetDefinition = "asset: " + chalk.cyan("./" + assetPath) + "; destination: " + chalk.cyan("./" + relativeOutputPath);

        var test = options.test;

        if (test) {
          var skip = _.isRegExp(test) ? !test.test(assetPath) : !test(assetPath);

          if (skip) {
            log(targetDefinition, chalk.yellow("[skipped; does not match test]"));

            return;
          }
        }

        removeFile(outputFilePath);

        var assetSize = asset.size();
        var assetSource = Array.isArray(asset.source()) ? asset.source().join("\n") : asset.source();

        if (options.useHashIndex) {
          var assetSourceHash = createHash("sha256").update(assetSource).digest("hex");

          if (assetSourceHashIndex[relativeOutputPath] && assetSourceHashIndex[relativeOutputPath] === assetSourceHash) {
            log(targetDefinition, chalk.yellow("[skipped; matched hash index]"));

            return;
          }

          assetSourceHashIndex[relativeOutputPath] = assetSourceHash;
        }

        mkdirp.sync(path.dirname(relativeOutputPath));

        try {
          if (options.basename && outputFilePath.indexOf("index.html") !== -1) {
            assetSource = assetSource.replace(/href="/g, `href="${options.basename}`).replace(/href='/g, `href='${options.basename}`).replace(/src="/g, `src="${options.basename}`).replace(/src='/g, `src='${options.basename}`);
          }
          fs.writeFileSync(relativeOutputPath.split("?")[0], assetSource);
          log(targetDefinition, chalk.green("[written]"), chalk.magenta("(" + filesize(assetSize) + ")"));
        } catch (error) {
          log(targetDefinition, chalk.bold.red("[is not written]"), chalk.magenta("(" + filesize(assetSize) + ")"));
          log(chalk.bold.bgRed("Exception:"), chalk.bold.red(error.message));
        }
      });
      // eslint-disable-next-line promise/prefer-await-to-callbacks
      callback();
    };

    /**
     * webpack 4+ comes with a new plugin system.
     *
     * Check for hooks in-order to support old plugin system
     */
    if (compiler.hooks) {
      compiler.hooks.afterEmit.tapAsync("write-file-webpack-plugin", handleAfterEmit);
    } else {
      compiler.plugin("after-emit", handleAfterEmit);
    }
  };

  return {
    apply
  };
};
//# sourceMappingURL=write-file.js.map